@@ -26,7 +26,7 @@ import sys
 from decimal import Context, Decimal, localcontext
 from fractions import Fraction
 from functools import reduce
-from inspect import isclass
+from inspect import isabstract, isclass
 from uuid import UUID
 
 import attr
@@ -69,6 +69,7 @@ from hypothesis.internal.reflection import (
     nicerepr,
     proxies,
     required_args,
+    reserved_means_kwonly_star,
 )
 from hypothesis.internal.validation import (
     check_type,
@@ -82,6 +83,7 @@ from hypothesis.internal.validation import (
 )
 from hypothesis.searchstrategy import SearchStrategy, check_strategy
 from hypothesis.searchstrategy.collections import (
+    FixedAndOptionalKeysDictStrategy,
     FixedKeysDictStrategy,
     ListStrategy,
     TupleStrategy,
@@ -96,11 +98,7 @@ from hypothesis.searchstrategy.datetime import (
 from hypothesis.searchstrategy.deferred import DeferredStrategy
 from hypothesis.searchstrategy.functions import FunctionStrategy
 from hypothesis.searchstrategy.lazy import LazyStrategy
-from hypothesis.searchstrategy.misc import (
-    BoolStrategy,
-    JustStrategy,
-    SampledFromStrategy,
-)
+from hypothesis.searchstrategy.misc import JustStrategy, SampledFromStrategy
 from hypothesis.searchstrategy.numbers import (
     BoundedIntStrategy,
     FixedBoundedFloatStrategy,
@@ -137,7 +135,7 @@ if False:
     import random  # noqa
     from types import ModuleType  # noqa
     from typing import Any, Dict, Union, Sequence, Callable, Pattern  # noqa
-    from typing import TypeVar, Tuple, List, Set, FrozenSet, overload  # noqa
+    from typing import TypeVar, Tuple, Iterable, List, Set, FrozenSet, overload  # noqa
     from typing import Type, Text, AnyStr, Optional  # noqa
 
     from hypothesis.utils.conventions import InferType  # noqa
@@ -345,7 +343,7 @@ def one_of(*args):
 @cacheable
 @defines_strategy_with_reusable_values
 def integers(min_value=None, max_value=None):
-    # type: (Real, Real) -> SearchStrategy[int]
+    # type: (int, int) -> SearchStrategy[int]
     """Returns a strategy which generates integers; in Python 2 these may be
     ints or longs.
 
@@ -425,7 +423,7 @@ def booleans():
     Examples from this strategy will shrink towards False (i.e.
     shrinking will try to replace True with False where possible).
     """
-    return BoolStrategy()
+    return sampled_from([False, True])
 
 
 @cacheable
@@ -854,7 +852,14 @@ class PrettyIter(object):
 
 
 @defines_strategy
-def iterables(elements, min_size=0, max_size=None, unique_by=None, unique=False):
+def iterables(
+    elements,  # type: SearchStrategy[Ex]
+    min_size=0,  # type: int
+    max_size=None,  # type: int
+    unique_by=None,  # type: Union[Callable, Tuple[Callable, ...]]
+    unique=False,  # type: bool
+):
+    # type: (...) -> SearchStrategy[Iterable[Ex]]
     """This has the same behaviour as lists, but returns iterables instead.
 
     Some iterables cannot be indexed (e.g. sets) and some do not have a
@@ -872,8 +877,13 @@ def iterables(elements, min_size=0, max_size=None, unique_by=None, unique=False)
 
 
 @defines_strategy
-def fixed_dictionaries(mapping):
-    # type: (Dict[T, SearchStrategy[Ex]]) -> SearchStrategy[Dict[T, Ex]]
+@reserved_means_kwonly_star
+def fixed_dictionaries(
+    mapping,  # type: Dict[T, SearchStrategy[Ex]]
+    __reserved=not_set,  # type: Any
+    optional=None,  # type: Dict[T, SearchStrategy[Ex]]
+):
+    # type: (...) -> SearchStrategy[Dict[T, Ex]]
     """Generates a dictionary of the same type as mapping with a fixed set of
     keys mapping to strategies. mapping must be a dict subclass.
 
@@ -886,8 +896,25 @@ def fixed_dictionaries(mapping):
     the generated dictionary.
     """
     check_type(dict, mapping, "mapping")
-    for v in mapping.values():
-        check_strategy(v)
+    for k, v in mapping.items():
+        check_strategy(v, "mapping[%r]" % (k,))
+    if __reserved is not not_set:
+        raise InvalidArgument("Do not pass __reserved; got %r" % (__reserved,))
+    if optional is not None:
+        check_type(dict, optional, "optional")
+        for k, v in optional.items():
+            check_strategy(v, "optional[%r]" % (k,))
+        if type(mapping) != type(optional):
+            raise InvalidArgument(
+                "Got arguments of different types: mapping=%s, optional=%s"
+                % (nicerepr(type(mapping)), nicerepr(type(optional)))
+            )
+        if set(mapping) & set(optional):
+            raise InvalidArgument(
+                "The following keys were in both mapping and optional, "
+                "which is invalid: %r" % (set(mapping) & set(optional))
+            )
+        return FixedAndOptionalKeysDictStrategy(mapping, optional)
     return FixedKeysDictStrategy(mapping)
 
 
@@ -1308,7 +1335,7 @@ def from_type(thing):
     naive time and datetime strategies.
 
     The resolution logic may be changed in a future version, but currently
-    tries these four options:
+    tries these five options:
 
     1. If ``thing`` is in the default lookup mapping or user-registered lookup,
        return the corresponding strategy.  The default lookup covers all types
@@ -1319,7 +1346,13 @@ def from_type(thing):
        the union of the strategies for those types that are not subtypes of
        other elements in the lookup.
     4. Finally, if ``thing`` has type annotations for all required arguments,
-       it is resolved via :func:`~hypothesis.strategies.builds`.
+       and is not an abstract class, it is resolved via
+       :func:`~hypothesis.strategies.builds`.
+    5. Because :mod:`abstract types <python:abc>` cannot be instantiated,
+       we treat abstract types as the union of their concrete subclasses.
+       Note that this lookup works via inheritance but not via
+       :obj:`~python:abc.ABCMeta.register`, so you may still need to use
+       :func:`~hypothesis.strategies.register_type_strategy`.
 
     There is a valuable recipe for leveraging ``from_type()`` to generate
     "everything except" values from a specified type. I.e.
@@ -1376,9 +1409,7 @@ def from_type(thing):
                 return one_of([from_type(t) for t in args])
         # We can't resolve forward references, and under Python 3.5 (only)
         # a forward reference is an instance of type.  Hence, explicit check:
-        elif (
-            hasattr(typing, "_ForwardRef") and type(thing) == typing._ForwardRef
-        ):  # pragma: no cover
+        elif type(thing) == getattr(typing, "_ForwardRef", None):  # pragma: no cover
             raise ResolutionFailed(
                 "thing=%s cannot be resolved.  Upgrading to python>=3.6 may "
                 "fix this problem via improvements to the typing module." % (thing,)
@@ -1406,7 +1437,7 @@ def from_type(thing):
     # not included because bool is a subclass of int as well as Number.
     strategies = [
         as_strategy(v, thing)
-        for k, v in types._global_type_lookup.items()
+        for k, v in sorted(types._global_type_lookup.items(), key=repr)
         if isinstance(k, type)
         and issubclass(k, thing)
         and sum(types.try_issubclass(k, typ) for typ in types._global_type_lookup) == 1
@@ -1432,7 +1463,15 @@ def from_type(thing):
             "using register_type_strategy" % (thing,)
         )
     # Finally, try to build an instance by calling the type object
-    return builds(thing)
+    if not isabstract(thing):
+        return builds(thing)
+    subclasses = thing.__subclasses__()
+    if not subclasses:
+        raise ResolutionFailed(
+            "Could not resolve %r to a strategy, because it is an abstract type "
+            "without any subclasses. Consider using register_type_strategy" % (thing,)
+        )
+    return sampled_from(subclasses).flatmap(from_type)
 
 
 @cacheable
@@ -2073,6 +2112,7 @@ class RunnerStrategy(SearchStrategy):
 
 @defines_strategy_with_reusable_values
 def runner(default=not_set):
+    # type: (Any) -> SearchStrategy[Any]
     """A strategy for getting "the current test runner", whatever that may be.
     The exact meaning depends on the entry point, but it will usually be the
     associated 'self' value for it.
@@ -2231,6 +2271,7 @@ def deferred(definition):
 
 @defines_strategy_with_reusable_values
 def emails():
+    # type: () -> SearchStrategy[Text]
     """A strategy for generating email addresses as unicode strings. The
     address format is specified in :rfc:`5322#section-3.4.1`. Values shrink
     towards shorter local-parts and host domains.
@@ -2248,14 +2289,14 @@ def emails():
     )
 
 
-# Mypy can't yet handle default values with generic types or typevars, but the
-# @overload workaround from https://github.com/python/mypy/issues/3737 doesn't
-# work with @composite functions - Mypy can't see that the function implements
-# `(Any, Callable[..., T], SearchStrategy[T]) -> Callable[..., T]`
-
-
 @defines_strategy
-def functions(like=lambda: None, returns=none()):
+def functions(
+    like=lambda: None,  # type: Callable[..., Any]
+    returns=none(),  # type: SearchStrategy[Any]
+):
+    # type: (...) -> SearchStrategy[Callable[..., Any]]
+    # The proper type signature of `functions()` would have T instead of Any, but mypy
+    # disallows default args for generics: https://github.com/python/mypy/issues/3737
     """A strategy for functions, which can be used in callbacks.
 
     The generated functions will mimic the interface of ``like``, which must
@@ -2280,6 +2321,7 @@ def functions(like=lambda: None, returns=none()):
 
 @composite
 def slices(draw, size):
+    # type: (Any, int) -> slice
     """Generates slices that will select indices up to the supplied size
 
     Generated slices will have start and stop indices that range from 0 to size - 1
