@@ -23,21 +23,23 @@ import numpy as np
 
 import hypothesis._strategies as st
 import hypothesis.internal.conjecture.utils as cu
-from hypothesis import Verbosity
+from hypothesis import Verbosity, assume
 from hypothesis._settings import note_deprecation
 from hypothesis.errors import InvalidArgument
-from hypothesis.internal.compat import hrange, integer_types
+from hypothesis.internal.compat import PY2, hrange, integer_types
 from hypothesis.internal.coverage import check_function
-from hypothesis.internal.reflection import proxies
+from hypothesis.internal.reflection import proxies, reserved_means_kwonly_star
 from hypothesis.internal.validation import check_type, check_valid_interval
 from hypothesis.reporting import current_verbosity
 from hypothesis.searchstrategy import SearchStrategy
+from hypothesis.utils.conventions import not_set
 
 if False:
     from typing import Any, Union, Sequence, Tuple, Optional  # noqa
     from hypothesis.searchstrategy.strategies import T  # noqa
 
     Shape = Tuple[int, ...]  # noqa
+    BasicIndex = Tuple[Union[int, slice, ellipsis, np.newaxis], ...]  # noqa
 
 TIME_RESOLUTIONS = tuple("Y  M  D  h  m  s  ms  us  ns  ps  fs  as".split())
 
@@ -94,7 +96,7 @@ def from_dtype(dtype):
             res = st.just(dtype.str.split("[")[-1][:-1])
         else:
             res = st.sampled_from(TIME_RESOLUTIONS)
-        result = st.builds(dtype.type, st.integers(-2 ** 63, 2 ** 63 - 1), res)
+        result = st.builds(dtype.type, st.integers(-(2 ** 63), 2 ** 63 - 1), res)
     else:
         raise InvalidArgument(u"No strategy inference for {}".format(dtype))
     return result.map(dtype.type)
@@ -133,42 +135,18 @@ class ArrayStrategy(SearchStrategy):
         self.element_strategy = element_strategy
         self.unique = unique
 
-        # Used by self.insert_element to check that the value can be stored
-        # in the array without e.g. overflowing.  See issues #1385 and #1591.
-        if dtype.kind in (u"i", u"u"):
-            self.check_cast = lambda x: np.can_cast(x, self.dtype, "safe")
-        elif dtype.kind == u"f" and dtype.itemsize == 2:
-            max_f2 = (2.0 - 2 ** -10) * 2 ** 15
-            self.check_cast = lambda x: (not np.isfinite(x)) or (-max_f2 <= x <= max_f2)
-        elif dtype.kind == u"f" and dtype.itemsize == 4:
-            max_f4 = (2.0 - 2 ** -23) * 2 ** 127
-            self.check_cast = lambda x: (not np.isfinite(x)) or (-max_f4 <= x <= max_f4)
-        elif dtype.kind == u"c" and dtype.itemsize == 8:
-            max_f4 = (2.0 - 2 ** -23) * 2 ** 127
-            self.check_cast = lambda x: (not np.isfinite(x)) or (
-                -max_f4 <= x.real <= max_f4 and -max_f4 <= x.imag <= max_f4
-            )
-        elif dtype.kind == u"U":
-            length = dtype.itemsize // 4
-            self.check_cast = lambda x: len(x) <= length and u"\0" not in x[length:]
-        elif dtype.kind in (u"S", u"a"):
-            self.check_cast = (
-                lambda x: len(x) <= dtype.itemsize and b"\0" not in x[dtype.itemsize :]
-            )
-        else:
-            self.check_cast = lambda x: True
-
     def set_element(self, data, result, idx, strategy=None):
         strategy = strategy or self.element_strategy
         val = data.draw(strategy)
         result[idx] = val
-        if self._report_overflow and not self.check_cast(val):
+        if self._report_overflow and val != result[idx] and val == val:
             note_deprecation(
                 "Generated array element %r from %r cannot be represented as "
-                "dtype %r - instead it becomes %r .  Consider using a more "
-                "precise strategy, as this will be an error in a future "
-                "version." % (val, strategy, self.dtype, result[idx]),
-                since="2018-10-25",
+                "dtype %r - instead it becomes %r (type %r).  Consider using a more "
+                "precise strategy, for example passing the `width` argument to "
+                "`floats()`, as this will be an error in a future version."
+                % (val, strategy, self.dtype, result[idx], type(result[idx])),
+                since="2019-07-28",
             )
             # Because the message includes the value of the generated element,
             # it would be easy to spam users with thousands of warnings.
@@ -180,7 +158,8 @@ class ArrayStrategy(SearchStrategy):
             return np.zeros(dtype=self.dtype, shape=self.shape)
 
         # Reset this flag for each test case to emit warnings from set_element
-        self._report_overflow = True
+        # Skip the check for object or void (multi-element) dtypes
+        self._report_overflow = self.dtype.kind not in ("O", "V")
 
         # This could legitimately be a np.empty, but the performance gains for
         # that would be so marginal that there's really not much point risking
@@ -610,20 +589,60 @@ def timedelta64_dtypes(max_period="Y", min_period="ns", endianness="?"):
 
 
 @defines_dtype_strategy
-def byte_string_dtypes(endianness="?", min_len=0, max_len=16):
+def byte_string_dtypes(endianness="?", min_len=1, max_len=16):
     # type: (str, int, int) -> st.SearchStrategy[np.dtype]
     """Return a strategy for generating bytestring dtypes, of various lengths
-    and byteorder."""
-    order_check("len", 0, min_len, max_len)
+    and byteorder.
+
+    While Hypothesis' string strategies can generate empty strings, string
+    dtypes with length 0 indicate that size is still to be determined, so
+    the minimum length for string dtypes is 1.
+    """
+    if min_len == 0:
+        note_deprecation(
+            "generating byte string dtypes for unspecified length ('S0') "
+            "is deprecated. min_len will be 1 instead.",
+            since="2019-09-09",
+        )
+        min_len = 1
+    if max_len == 0:
+        note_deprecation(
+            "generating byte string dtypes for unspecified length ('S0') "
+            "is deprecated. max_len will be 1 instead.",
+            since="2019-09-09",
+        )
+        max_len = 1
+
+    order_check("len", 1, min_len, max_len)
     return dtype_factory("S", list(range(min_len, max_len + 1)), None, endianness)
 
 
 @defines_dtype_strategy
-def unicode_string_dtypes(endianness="?", min_len=0, max_len=16):
+def unicode_string_dtypes(endianness="?", min_len=1, max_len=16):
     # type: (str, int, int) -> st.SearchStrategy[np.dtype]
     """Return a strategy for generating unicode string dtypes, of various
-    lengths and byteorder."""
-    order_check("len", 0, min_len, max_len)
+    lengths and byteorder.
+
+    While Hypothesis' string strategies can generate empty strings, string
+    dtypes with length 0 indicate that size is still to be determined, so
+    the minimum length for string dtypes is 1.
+    """
+    if min_len == 0:
+        note_deprecation(
+            "generating unicode string dtypes for unspecified length ('U0') "
+            "is deprecated. min_len will be 1 instead.",
+            since="2019-09-09",
+        )
+        min_len = 1
+    if max_len == 0:
+        note_deprecation(
+            "generating unicode string dtypes for unspecified length ('U0') "
+            "is deprecated. max_len will be 1 instead.",
+            since="2019-09-09",
+        )
+        max_len = 1
+
+    order_check("len", 1, min_len, max_len)
     return dtype_factory("U", list(range(min_len, max_len + 1)), None, endianness)
 
 
@@ -638,11 +657,15 @@ def array_dtypes(
     """Return a strategy for generating array (compound) dtypes, with members
     drawn from the given subtype strategy."""
     order_check("size", 0, min_size, max_size)
-    native_strings = st.from_type(str).filter(bool)  # See issue #1798 re: filter!
-    elements = st.tuples(native_strings, subtype_strategy)
+    # Field names must be native strings and the empty string is weird; see #1963.
+    if PY2:
+        field_names = st.binary(min_size=1)
+    else:
+        field_names = st.text(min_size=1)
+    elements = st.tuples(field_names, subtype_strategy)
     if allow_subarrays:
         elements |= st.tuples(
-            native_strings, subtype_strategy, array_shapes(max_dims=2, max_side=2)
+            field_names, subtype_strategy, array_shapes(max_dims=2, max_side=2)
         )
     return st.lists(
         elements=elements,
@@ -770,14 +793,13 @@ def broadcastable_shapes(shape, min_dims=0, max_dims=None, min_side=1, max_side=
     with the provided shape.
 
     Examples from this strategy shrink towards a shape with length ``min_dims``.
-    The size of an aligned dimension shrinks towards being a singleton. The
+    The size of an aligned dimension shrinks towards size ``1``. The
     size of an unaligned dimension shrink towards ``min_side``.
 
     * ``shape`` a tuple of integers
     * ``min_dims`` The smallest length that the generated shape can possess.
     * ``max_dims`` The largest length that the generated shape can possess.
-      shape can possess. Cannot exceed 32. The default-value for ``max_dims``
-      is ``2 + max(len(shape), min_dims)``.
+      The default-value for ``max_dims`` is ``min(32, max(len(shape), min_dims) + 2)``.
     * ``min_side`` The smallest size that an unaligned dimension can possess.
     * ``max_side`` The largest size that an unaligned dimension can possess.
       The default value is 2 + 'size-of-largest-aligned-dimension'.
@@ -796,7 +818,7 @@ def broadcastable_shapes(shape, min_dims=0, max_dims=None, min_side=1, max_side=
     check_type(integer_types, min_dims, "min_dims")
 
     if max_dims is None:
-        max_dims = max(len(shape), min_dims) + 2
+        max_dims = min(32, max(len(shape), min_dims) + 2)
     else:
         check_type(integer_types, max_dims, "max_dims")
 
@@ -850,6 +872,122 @@ def broadcastable_shapes(shape, min_dims=0, max_dims=None, min_side=1, max_side=
     )
 
 
+class BasicIndexStrategy(SearchStrategy):
+    def __init__(self, shape, min_dims, max_dims, allow_ellipsis, allow_newaxis):
+        assert 0 <= min_dims <= max_dims <= 32
+        SearchStrategy.__init__(self)
+        self.shape = shape
+        self.min_dims = min_dims
+        self.max_dims = max_dims
+        self.allow_ellipsis = allow_ellipsis
+        self.allow_newaxis = allow_newaxis
+
+    def do_draw(self, data):
+        # General plan: determine the actual selection up front with a straightforward
+        # approach that shrinks well, then complicate it by inserting other things.
+        result = []
+        for dim_size in self.shape:
+            if dim_size == 0:
+                result.append(slice(None))
+                continue
+            strategy = st.integers(-dim_size, dim_size - 1) | st.slices(dim_size)
+            result.append(data.draw(strategy))
+        # Insert some number of new size-one dimensions if allowed
+        result_dims = sum(isinstance(idx, slice) for idx in result)
+        while (
+            self.allow_newaxis
+            and result_dims < self.max_dims
+            and (result_dims < self.min_dims or data.draw(st.booleans()))
+        ):
+            result.insert(data.draw(st.integers(0, len(result))), np.newaxis)
+            result_dims += 1
+        # Check that we'll have the right number of dimensions; reject if not.
+        # It's easy to do this by construction iff you don't care about shrinking,
+        # which is really important for array shapes.  So we filter instead.
+        assume(self.min_dims <= result_dims <= self.max_dims)
+        # This is a quick-and-dirty way to insert ..., xor shorten the indexer,
+        # but it means we don't have to do any structural analysis.
+        if self.allow_ellipsis and data.draw(st.booleans()):
+            # Choose an index; then replace all adjacent whole-dimension slices.
+            i = j = data.draw(st.integers(0, len(result)))
+            while i > 0 and result[i - 1] == slice(None):
+                i -= 1
+            while j < len(result) and result[j] == slice(None):
+                j += 1
+            result[i:j] = [Ellipsis]
+        else:
+            while result[-1:] == [slice(None, None)] and data.draw(st.integers(0, 7)):
+                result.pop()
+        return tuple(result)
+
+
+@st.defines_strategy
+@reserved_means_kwonly_star
+def basic_indices(
+    shape,
+    __reserved=not_set,
+    min_dims=0,
+    max_dims=None,
+    allow_newaxis=False,
+    allow_ellipsis=True,
+):
+    # type: (Shape, Any, int, int, bool, bool) -> st.SearchStrategy[BasicIndex]
+    """
+    The ``basic_indices`` strategy generates `basic indexes
+    <https://docs.scipy.org/doc/numpy/reference/arrays.indexing.html>`__  for
+    arrays of the specified shape, which may include dimensions of size zero.
+
+    It generates tuples containing some mix of integers, :obj:`python:slice` objects,
+    ``...`` (Ellipsis), and :obj:`numpy:numpy.newaxis`; which when used to index a
+    ``shape``-shaped array will produce either a scalar or a shared-memory view.
+
+    * ``shape``: the array shape that will be indexed, as a tuple of integers >= 0.
+      This must be at least two-dimensional for a tuple to be a valid basic index;
+      for one-dimensional arrays use :func:`~hypothesis.strategies.slices` instead.
+    * ``min_dims``: the minimum dimensionality of the resulting view from use of
+      the generated index.  When ``min_dims == 0``, scalars and zero-dimensional
+      arrays are both allowed.
+    * ``max_dims``: the maximum dimensionality of the resulting view.
+      If not specified, it defaults to ``max(len(shape), min_dims) + 2``.
+    * ``allow_ellipsis``: whether ``...``` is allowed in the index.
+    * ``allow_newaxis``: whether :obj:`numpy:numpy.newaxis` is allowed in the index.
+
+    Note that the length of the generated tuple may be anywhere between zero
+    and ``min_dims``.  It may not match the length of ``shape``, or even the
+    dimensionality of the array view resulting from its use!
+    """
+    # Arguments to exclude scalars, zero-dim arrays, and dims of size zero were
+    # all considered and rejected.  We want users to explicitly consider those
+    # cases if they're dealing in general indexers, and while it's fiddly we can
+    # back-compatibly add them later (hence using __reserved to sim kwonlyargs).
+    check_type(tuple, shape, "shape")
+    if __reserved is not not_set:
+        raise InvalidArgument("Do not pass the __reserved argument.")
+    check_type(bool, allow_ellipsis, "allow_ellipsis")
+    check_type(bool, allow_newaxis, "allow_newaxis")
+    check_type(integer_types, min_dims, "min_dims")
+    if max_dims is None:
+        max_dims = min(max(len(shape), min_dims) + 2, 32)
+    else:
+        check_type(integer_types, max_dims, "max_dims")
+    order_check("dims", 0, min_dims, max_dims)
+    check_argument(
+        max_dims <= 32,
+        "max_dims=%r, but numpy arrays have at most 32 dimensions" % (max_dims,),
+    )
+    check_argument(
+        all(isinstance(x, integer_types) and x >= 0 for x in shape),
+        "shape=%r, but all dimensions must be of integer size >= 0" % (shape,),
+    )
+    return BasicIndexStrategy(
+        shape,
+        min_dims=min_dims,
+        max_dims=max_dims,
+        allow_ellipsis=allow_ellipsis,
+        allow_newaxis=allow_newaxis,
+    )
+
+
 @st.defines_strategy
 def integer_array_indices(shape, result_shape=array_shapes(), dtype="int"):
     # type: (Shape, SearchStrategy[Shape], np.dtype) -> st.SearchStrategy[Tuple[np.ndarray, ...]]
